@@ -31,6 +31,7 @@
 #include "ui.h"
 #include "mp-equation.h"
 #include "register.h"
+#include "currency.h"
 
 static GCDisplayState *
 get_state(GCDisplay *display)
@@ -39,34 +40,6 @@ get_state(GCDisplay *display)
 }
 
 
-static char *
-str_replace(char *str, char *from, char *to)
-{
-    char output[MAX_DISPLAY];
-    int offset = 0;
-    char *c;
-    int flen = strlen(from);
-    int tlen = strlen(to);
-
-    for (c = str; *c && offset < MAX_DISPLAY - 1; c++, offset++) {
-        if (strncasecmp(from, c, flen) == 0) {
-            SNPRINTF(output + offset, MAX_DISPLAY - offset, "%s", to);
-            c += flen - 1;
-            offset += tlen - 1;
-        } else {
-            output[offset] = *c;
-        }
-    }
-
-    if (offset >= MAX_DISPLAY)
-        offset = MAX_DISPLAY - 1;
-    output[offset] = '\0';
-
-    free(str);
-
-    return strdup(output);
-}
-
 /* Add in the thousand separators characters if required and if we are
  * currently in the decimal numeric base, use the "right" radix character.
  */
@@ -139,43 +112,60 @@ localize_expression(GCDisplay *display, char *dest, const char *src, int dest_le
 void
 display_clear(GCDisplay *display)
 {
+    GCDisplayState *state;
+
+    state = get_state(display);
     display_set_string(display, "", -1);
+    state->ans_start = -1;
+    state->ans_end = -1;
 }
 
 
-const char *
-display_get_text(GCDisplay *display)
+static const char *
+get_text(GCDisplay *display)
 {
     return get_state(display)->expression;
 }
 
 
-gboolean display_get_integer(GCDisplay *display, gint64 *value)
+static char *
+get_expression(GCDisplay *display)
+{
+    GCDisplayState *state;
+
+    state = get_state(display);
+    if(state->ans_start >= 0)
+        return g_strdup_printf("%.*sans%s", state->ans_start, state->expression, g_utf8_offset_to_pointer(state->expression, state->ans_end));
+    else
+        return g_strdup(state->expression);
+}
+
+
+gboolean
+display_get_integer(GCDisplay *display, gint64 *value)
 {
-    const char *text;
-    char buf[MAX_DISPLAY];
     gchar *endptr;
 
-    text = display_get_text(display);
     if (display_is_result(display)) {
-        display_make_number(display, buf, MAX_DISPLAY, display_get_answer(display));
-        text = buf;
+        *value = mp_cast_to_int(display_get_answer(display));
+        return TRUE;
     }
 
-    *value = g_ascii_strtoll(text, &endptr, 10);
+    *value = g_ascii_strtoll(get_text(display), &endptr, 10);
     if(*endptr != '\0' || ((*value == G_MAXINT64 || *value == G_MININT64) && errno == ERANGE))
         return FALSE;
     return TRUE;
 }
 
 
-gboolean display_get_unsigned_integer(GCDisplay *display, guint64 *value)
+gboolean
+display_get_unsigned_integer(GCDisplay *display, guint64 *value)
 {
     const char *text;
     char buf[MAX_DISPLAY];
     gchar *endptr;
 
-    text = display_get_text(display);
+    text = get_text(display);
     if (display_is_result(display)) {
         display_make_number(display, buf, MAX_DISPLAY, display_get_answer(display));
         text = buf;
@@ -192,7 +182,8 @@ gboolean display_get_unsigned_integer(GCDisplay *display, guint64 *value)
 }
 
 
-MPNumber *display_get_answer(GCDisplay *display)
+MPNumber *
+display_get_answer(GCDisplay *display)
 {
     return &get_state(display)->ans;
 }
@@ -212,6 +203,7 @@ display_set_number(GCDisplay *display, const MPNumber *x)
     char text[MAX_DISPLAY];
     int enabled;
     guint64 bit_value;
+
     display_make_number(display, text, MAX_DISPLAY, x);
     display_set_string(display, text, -1);
 
@@ -223,7 +215,14 @@ display_set_number(GCDisplay *display, const MPNumber *x)
 void
 display_set_answer(GCDisplay *display)
 {
-    display_set_string(display, "ans", -1);
+    GCDisplayState *state;
+    char text[MAX_DISPLAY];
+
+    state = get_state(display);
+    display_make_number(display, text, MAX_DISPLAY, &state->ans);
+    display_set_string(display, text, -1);
+    state->ans_start = 0;
+    state->ans_end = g_utf8_strlen(text, -1);
 }
 
 
@@ -322,7 +321,8 @@ update_undo_redo_button_sensitivity(GCDisplay *display)
 }
 
 
-void display_clear_stack(GCDisplay *display)
+void
+display_clear_stack(GCDisplay *display)
 {
     int i = display->h.begin;
     while (i != display->h.end) {
@@ -337,7 +337,8 @@ void display_clear_stack(GCDisplay *display)
 }
 
 
-void display_push(GCDisplay *display)
+void
+display_push(GCDisplay *display)
 {
     int c;
 
@@ -347,7 +348,9 @@ void display_push(GCDisplay *display)
         do {
             i = ((i + 1) % UNDO_HISTORY_LENGTH);
             free(display->h.e[i].expression);
-            display->h.e[i].expression = strdup("ans");
+            display->h.e[i].expression = strdup("ans"); // FIXME: Use actual number
+            display->h.e[i].ans_start = -1;
+            display->h.e[i].ans_end = -1;
         } while (i != display->h.end);
     }
 
@@ -366,7 +369,8 @@ void display_push(GCDisplay *display)
 }
 
 
-void display_pop(GCDisplay *display)
+void
+display_pop(GCDisplay *display)
 {
     if (display->h.current != display->h.begin) {
         display->h.current = ((display->h.current - 1) % UNDO_HISTORY_LENGTH);
@@ -405,10 +409,19 @@ display_is_undo_step(GCDisplay *display)
 void
 display_insert(GCDisplay *display, int cursor_start, int cursor_end, const char *text)
 {
+    GCDisplayState *state;   
     char buf[MAX_DISPLAY];
 
+    state = get_state(display);
+
+    /* If inside ans variable then modify number */
+    if (state->ans_start >= 0 && cursor_start >= state->ans_start && cursor_start <= state->ans_end) {
+        state->ans_start = -1;
+        state->ans_end = -1;
+    }
+
     if (cursor_start < 0) {
-        SNPRINTF(buf, MAX_DISPLAY, "%s%s", display_get_text(display), text);
+        SNPRINTF(buf, MAX_DISPLAY, "%s%s", get_text(display), text);
         display_set_string(display, buf, -1);
     } else {
         GString *new_text;
@@ -467,8 +480,6 @@ display_insert_number(GCDisplay *display, int cursor_start, int cursor_end, cons
 void
 display_backspace(GCDisplay *display, int cursor_start, int cursor_end)
 {
-    char buf[MAX_DISPLAY] = "";
-    GCDisplayState *e = get_state(display);
     int cursor;
 
     /* Can't delete empty display */
@@ -480,14 +491,7 @@ display_backspace(GCDisplay *display, int cursor_start, int cursor_end)
     /* If cursor is at end of the line then delete the last character preserving accuracy */
     if (cursor_start < 0) {
         int len;
-
         len = g_utf8_strlen(ui_get_display(), -1);
-
-        if (display_is_result(display)) {
-            display_make_number(display, buf, MAX_DISPLAY, &e->ans);
-            e->expression = str_replace(e->expression, "ans", buf);
-        }
-
         display_insert(display, len - 1, len, "");
     } else if (cursor_start != cursor_end) {
         display_insert(display, cursor_start, cursor_end, "");
@@ -508,27 +512,20 @@ display_delete(GCDisplay *display, int cursor_start, int cursor_end)
 }
 
 
-void
-display_surround(GCDisplay *display, const char *prefix, const char *suffix)
-{
-    char buffer[MAX_DISPLAY];
-
-    SNPRINTF(buffer, MAX_DISPLAY, "%s%s%s", prefix, display_get_text(display), suffix);
-    display_set_string(display, buffer, -1);
-}
-
-
 gboolean
 display_is_empty(GCDisplay *display)
 {
-    return strcmp(display_get_text(display), "") == 0;
+    return strcmp(get_text(display), "") == 0;
 }
 
 
 gboolean
 display_is_result(GCDisplay *display)
 {
-    if (strcmp(display_get_text(display), "ans") == 0)
+    GCDisplayState *state;
+
+    state = get_state(display);
+    if (state->ans_start == 0 && state->ans_end == g_utf8_strlen(state->expression, -1))
         return TRUE;
 
     return FALSE;
@@ -545,7 +542,7 @@ display_is_usable_number(GCDisplay *display, MPNumber *z)
         mp_set_from_mp(display_get_answer(display), z);
         return TRUE;
     } else {
-        return mp_set_from_string(display_get_text(display), z) == 0;
+        return mp_set_from_string(get_text(display), z) == 0;
     }
 }
 
@@ -564,12 +561,16 @@ display_init(GCDisplay *display)
     display->word_size = 32;
     display->angle_unit = MP_DEGREES;
 
-    for (i = 0; i < UNDO_HISTORY_LENGTH; i++)
+    for (i = 0; i < UNDO_HISTORY_LENGTH; i++) {
         display->h.e[i].expression = strdup("");
+        display->h.e[i].ans_start = -1;
+        display->h.e[i].ans_end = -1;
+    }
 }
 
 
-void display_set_accuracy(GCDisplay *display, int accuracy)
+void
+display_set_accuracy(GCDisplay *display, int accuracy)
 {
     display->accuracy = accuracy;
     get_state(display)->cursor = -1;
@@ -577,7 +578,8 @@ void display_set_accuracy(GCDisplay *display, int accuracy)
 }
 
 
-void display_set_show_thousands_separator(GCDisplay *display, gboolean visible)
+void
+display_set_show_thousands_separator(GCDisplay *display, gboolean visible)
 {
     display->show_tsep = visible;
     display_set_cursor(display, -1);
@@ -585,7 +587,8 @@ void display_set_show_thousands_separator(GCDisplay *display, gboolean visible)
 }
 
 
-void display_set_show_trailing_zeroes(GCDisplay *display, gboolean visible)
+void
+display_set_show_trailing_zeroes(GCDisplay *display, gboolean visible)
 {
     display->show_zeroes = visible;
     get_state(display)->cursor = -1;
@@ -593,32 +596,38 @@ void display_set_show_trailing_zeroes(GCDisplay *display, gboolean visible)
 }
 
 
-void display_set_format(GCDisplay *display, DisplayFormat format)
+void
+display_set_format(GCDisplay *display, DisplayFormat format)
 {
     display->format = format;
     get_state(display)->cursor = -1;
     display_refresh(display);
 }
 
-void display_set_word_size(GCDisplay *display, int word_size)
+
+void
+display_set_word_size(GCDisplay *display, int word_size)
 {
     display->word_size = word_size;
 }
 
-void display_set_angle_unit(GCDisplay *display, MPAngleUnit angle_unit)
+
+void
+display_set_angle_unit(GCDisplay *display, MPAngleUnit angle_unit)
 {
     display->angle_unit = angle_unit;
 }
 
+
 /* Convert engineering or scientific number in the given base. */
 static void
 make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *x, int base_)
 {
-    static char digits[] = "0123456789ABCDEF";
-    char fixed[MAX_DIGITS];
+    char fixed[MAX_DIGITS], *c;
     MPNumber t, z, base, base3, base10, base10inv, mantissa;
-    int ddig, eng, exponent = 0;
+    int eng, exponent = 0;
     GString *string;
+    const char *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
 
     string = g_string_sized_new(target_len);
 
@@ -661,30 +670,14 @@ make_eng_sci(GCDisplay *display, char *target, int target_len, const MPNumber *x
 
     mp_cast_to_string(&mantissa, base_, display->accuracy, !display->show_zeroes, fixed, MAX_DIGITS);
     g_string_append(string, fixed);
-    g_string_append(string, "×10^");
-
+    g_string_append_printf(string, "×10");
     if (exponent < 0) {
         exponent = -exponent;
-        g_string_append(string, "−");
-    } else {
-        g_string_append(string, "+");
-    }
-
-    mp_set_from_string("0.5", &t);
-    mp_add_integer(&t, exponent, &z);
-    mp_set_from_integer(1, &t);
-    for (ddig = 0; mp_is_greater_equal(&z, &t); ddig++) {
-        mp_divide(&z, &base, &z);
-    }
-
-    while (ddig-- > 0) {
-        int dval;
-
-        mp_multiply(&z, &base, &z);
-        dval = mp_cast_to_int(&z);
-        g_string_append_c(string, digits[dval]);
-        mp_add_integer(&z, -dval, &z);
+        g_string_append(string, "⁻");
     }
+    snprintf(fixed, MAX_DIGITS, "%d", exponent);
+    for (c = fixed; *c; c++)
+        g_string_append(string, super_digits[*c - '0']);
 
     strncpy(target, string->str, target_len);
     g_string_free(string, TRUE);
@@ -723,6 +716,7 @@ get_variable(const char *name, MPNumber *z, void *data)
     char *c, *lower_name;
     int result = 1;
     GCDisplay *display = data;
+    MPNumber *t;
 
     lower_name = strdup(name);
     for (c = lower_name; *c; c++)
@@ -730,12 +724,15 @@ get_variable(const char *name, MPNumber *z, void *data)
 
     if (strcmp(lower_name, "rand") == 0)
         mp_set_from_random(z);
-    else if (lower_name[0] == 'r')
-        mp_set_from_mp(register_get_value(atoi(name+1)), z);
     else if (strcmp(lower_name, "ans") == 0)
         mp_set_from_mp(display_get_answer(display), z);
-    else
-        result = 0;
+    else {
+        t = register_get_value(name);
+        if (t)
+            mp_set_from_mp(t, z);
+        else
+            result = 0;
+    }
 
     free(lower_name);
 
@@ -747,8 +744,176 @@ static void
 set_variable(const char *name, const MPNumber *x, void *data)
 {
     /* FIXME: Don't allow writing to built-in variables, e.g. ans, rand, sin, ... */
-    if (name[0] == 'R' || name[0] == 'r')
-        register_set_value(atoi(name+1), x);
+    register_set_value(name, x);
+}
+
+
+static int
+do_convert(const char *units[][2], const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z)
+{
+    int x_index, z_index;
+    MPNumber x_factor, z_factor;
+    
+    for (x_index = 0; units[x_index][0] != NULL && strcmp(units[x_index][0], x_units) != 0; x_index++);
+    if (units[x_index][0] == NULL)
+        return 0;
+    for (z_index = 0; units[z_index][0] != NULL && strcmp(units[z_index][0], z_units) != 0; z_index++);
+    if (units[z_index][0] == NULL)
+        return 0;
+
+    mp_set_from_string(units[x_index][1], &x_factor);
+    mp_set_from_string(units[z_index][1], &z_factor);
+    mp_multiply(x, &x_factor, z);
+    mp_divide(z, &z_factor, z);
+
+    return 1;
+}
+
+
+static int
+convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z, void *data)
+{
+    const char *length_units[][2] = {
+        {"parsec",     "30857000000000000"},
+        {"parsecs",    "30857000000000000"},
+        {"pc",         "30857000000000000"},
+        {"lightyear",   "9460730472580800"},
+        {"lightyears",  "9460730472580800"},
+        {"ly",          "9460730472580800"},
+        {"au",              "149597870691"},
+        {"nm",                   "1852000"},
+        {"mile",                    "1609.344"},
+        {"miles",                   "1609.344"},
+        {"m",                       "1609.344"},
+        {"kilometer",               "1000"},
+        {"kilometers",              "1000"},
+        {"km",                      "1000"},
+        {"kms",                     "1000"},
+        {"cable",                    "219.456"},
+        {"cables",                   "219.456"},
+        {"cb",                       "219.456"},
+        {"fathom",                     "1.8288"},
+        {"fathoms",                    "1.8288"},
+        {"ftm",                        "1.8288"},
+        {"meter",                      "1"},
+        {"meters",                     "1"},
+        {"m",                          "1"},
+        {"yard",                       "0.9144"},
+        {"yd",                         "0.9144"},
+        {"foot",                       "0.3048"},
+        {"feet",                       "0.3048"},
+        {"ft",                         "0.3048"},
+        {"inch",                       "0.0254"},
+        {"inches",                     "0.0254"},
+        {"centimeter",                 "0.01"},
+        {"centimeters",                "0.01"},
+        {"cm",                         "0.01"},
+        {"cms",                        "0.01"},
+        {"millimeter",                 "0.001"},
+        {"millimeters",                "0.001"},
+        {"mm",                         "0.001"},
+        {"micrometer",                 "0.000001"},
+        {"micrometers",                "0.000001"},
+        {"um",                         "0.000001"},
+        {"nanometer",                  "0.000000001"},
+        {"nanometers",                 "0.000000001"},
+        {NULL, NULL}
+    };
+
+    const char *area_units[][2] = {
+        {"hectare",         "10000"},
+        {"hectares",        "10000"},
+        {"acre",             "4046.8564224"},
+        {"acres",            "4046.8564224"},
+        {"m²",                  "1"},
+        {"cm²",                 "0.001"},
+        {NULL, NULL}
+    };
+
+    const char *volume_units[][2] = {
+        {"cm³",              "1000"},
+        {"gallon",              "3.785412"},
+        {"gallons",             "3.785412"},
+        {"gal",                 "3.785412"},
+        {"litre",               "1"},
+        {"litres",              "1"},
+        {"liter",               "1"},
+        {"liters",              "1"},
+        {"L",                   "1"},
+        {"quart",               "0.9463529"},
+        {"quarts",              "0.9463529"},
+        {"qt",                  "0.9463529"},
+        {"pint",                "0.4731765"},
+        {"pints",               "0.4731765"},
+        {"pt",                  "0.4731765"},
+        {"millilitre",          "0.001"},
+        {"millilitres",         "0.001"},
+        {"milliliter",          "0.001"},
+        {"milliliters",         "0.001"},
+        {"mL",                  "0.001"},
+        {"m³",                  "0.001"},
+        {NULL, NULL}
+    };
+
+    const char *weight_units[][2] = {
+        {"tonne",            "1000"},
+        {"tonnes",           "1000"},
+        {"kilograms",           "1"},
+        {"kilogramme",          "1"},
+        {"kilogrammes",         "1"},
+        {"kg",                  "1"},
+        {"kgs",                 "1"},
+        {"pound",               "0.45359237"},
+        {"pounds",              "0.45359237"},
+        {"lb",                  "0.45359237"},
+        {"ounce",               "0.002834952"},
+        {"ounces",              "0.002834952"},
+        {"oz",                  "0.002834952"},
+        {"gram",                "0.001"},
+        {"grams",               "0.001"},
+        {"gramme",              "0.001"},
+        {"grammes",             "0.001"},
+        {"g",                   "0.001"},
+        {NULL, NULL}
+    };
+    
+    const char *time_units[][2] = {
+        {"year",         "31557600"},
+        {"years",        "31557600"},
+        {"day",             "86400"},
+        {"days",            "86400"},
+        {"hour",             "3600"},
+        {"hours",            "3600"},
+        {"minute",             "60"},
+        {"minutes",            "60"},
+        {"second",              "1"},
+        {"seconds",             "1"},
+        {"s",                   "1"},
+        {"millisecond",         "0.001"},
+        {"milliseconds",        "0.001"},
+        {"ms",                  "0.001"},
+        {"microsecond",         "0.000001"},
+        {"microseconds",        "0.000001"},
+        {"us",                  "0.000001"},
+        {NULL, NULL}
+    };
+    
+    /* See if currency */
+    currency_load_rates();
+    if (currency_get_index(x_units) >= 0 && currency_get_index(z_units) >= 0)
+    {
+        currency_convert(x, currency_get_index(x_units), currency_get_index(z_units), z);
+        return 1;
+    }
+    
+    if (do_convert(length_units, x, x_units, z_units, z) ||
+        do_convert(area_units, x, x_units, z_units, z) ||
+        do_convert(volume_units, x, x_units, z_units, z) ||
+        do_convert(weight_units, x, x_units, z_units, z) ||
+        do_convert(time_units, x, x_units, z_units, z))
+        return 1;
+
+    return 0;
 }
 
 
@@ -762,6 +927,7 @@ parse(GCDisplay *display, const char *text, MPNumber *z, char **error_token)
     options.angle_units = display->angle_unit;
     options.get_variable = get_variable;
     options.set_variable = set_variable;
+    options.convert = convert;
     options.callback_data = display;
 
     return mp_equation_parse(text, &options, z, error_token);
@@ -860,7 +1026,7 @@ do_shift(GCDisplay *display, int count)
     if (!display_is_usable_number(display, &z)) {
         /* Translators: This message is displayed in the status bar when a bit
            shift operation is performed and the display does not contain a number */
-        ui_set_statusbar(_("No sane value to do bitwise shift"));
+        ui_set_statusbar(_("No sane value to bitwise shift"));
     }
     else {
         mp_shift(&z, count, display_get_answer(display));
@@ -897,21 +1063,20 @@ do_factorize()
 
 
 static void
-do_sto(GCDisplay *display, int index)
+do_sto(GCDisplay *display, const char *name)
 {
-    MPNumber temp;
+    MPNumber t;
 
-    if (!display_is_usable_number(display, &temp))
+    if (!display_is_usable_number(display, &t))
         ui_set_statusbar(_("No sane value to store"));
     else
-        register_set_value(index, &temp);
+        register_set_value(name, &t);
 }
 
 
 void
 display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_start, int cursor_end)
 {
-    char buf[MAXLINE];
     MPNumber *ans;
     int enabled;
     guint64 bit_value;
@@ -959,12 +1124,11 @@ display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_s
             return;
 
         case FN_STORE:
-            do_sto(display, GPOINTER_TO_INT (arg));
+            do_sto(display, (const char *)arg);
             return;
 
         case FN_RECALL:
-            SNPRINTF(buf, MAXLINE, "R%s", (const char *)arg);
-            display_insert(display, cursor_start, cursor_end, buf);
+            display_insert(display, cursor_start, cursor_end, (const char *)arg);
             break;
 
         case FN_BACKSPACE:
@@ -1009,12 +1173,15 @@ display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_s
                 MPNumber z;
                 int result;
                 const char *message = NULL;
-                char *error_token;
+                char *text, *error_token;
 
+                text = get_expression (display);
                 result = parse(display,
-                               display_get_text(display),
+                               text,
                                &z,
                                &error_token);
+                g_free(text);
+
                 switch (result) {
                     case PARSER_ERR_NONE:
                         mp_set_from_mp(&z, ans);
@@ -1038,6 +1205,11 @@ display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_s
                         free(error_token);
                         break;
 
+                    case PARSER_ERR_UNKNOWN_CONVERSION:
+                        /* Translators: Error displayed to user when an conversion with unknown units is attempted */
+                        message = g_strdup_printf(_("Unknown conversion"));
+                        break;
+
                     case PARSER_ERR_MP:
                         message = mp_get_error();
                         break;
@@ -1053,6 +1225,10 @@ display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_s
             break;
 
         case FN_TEXT:
+            /* Start new equation when entering digits after existing result */
+            if(display_is_result(display) && g_unichar_isdigit(g_utf8_get_char((char*)arg)))
+                display_clear(display);
+
             display_insert(display, cursor_start, cursor_end, (const char *)arg);
             break;
     }
